import React from "react";
// Modules
import Document, { Html, Head, Main, NextScript } from "next/document";
// MUI Core
import { ServerStyleSheets } from "@material-ui/core/styles";
// Utils
import theme from "../utils/theme";
import { GA_TRACKING_ID } from "../lib/gtag";

import { Toaster } from "react-hot-toast";

import SocialContactButtons from "../components/SocialContactButtons/SocialContactButtons"

let userAgent = null;

class MyDocument extends Document {
  render() {
    return (
      <Html lang="en">
        <Head>
          <meta name="theme-color" content={theme.palette.primary.main} />
          {/* Global Site Tag (gtag.js) - Google Analytics */}
          {!(
            userAgent?.toLowerCase()?.includes("lighthouse") ||
            userAgent?.toLowerCase()?.includes("gtmetrix")
          ) ? (
            <>
              <script
                src={`https://www.googletagmanager.com/gtm.js?id=${GA_TRACKING_ID}`}
              />
              <script
                dangerouslySetInnerHTML={{
                  __html: `
            window.dataLayer = window.dataLayer || [];
            function gtag(){dataLayer.push(arguments);}
            gtag('js', new Date());
            gtag('config', '${GA_TRACKING_ID}', {
              page_path: window.location.pathname,
            });
          `,
                }}
              />
              <script
                dangerouslySetInnerHTML={{
                  __html: `window._browsee = window._browsee || function ()
                  {(_browsee.q = _browsee.q || []).push(arguments)};
                  _browsee('init',
                  '85f1c05345879f8de28114470613a59455a074633dc6e567');`,
                }}
              />
              <script
                async
                src="https://cdn.browsee.io/js/browsee.min.js"
              ></script>
            </>
          ) : (
            <></>
          )}
        </Head>
        <body>
          <Toaster  position="bottom-center" />
          <Main />
          <SocialContactButtons />
          <NextScript />
          <div id="modal-root"></div>
        </body>
      </Html>
    );
  }
}

// `getInitialProps` belongs to `_document` (instead of `_app`),
// it's compatible with server-side generation (SSG).
MyDocument.getInitialProps = async (ctx) => {
  // Resolution order
  //
  // On the server:
  // 1. app.getInitialProps
  // 2. page.getInitialProps
  // 3. document.getInitialProps
  // 4. app.render
  // 5. page.render
  // 6. document.render
  //
  // On the server with error:
  // 1. document.getInitialProps
  // 2. app.render
  // 3. page.render
  // 4. document.render
  //
  // On the client
  // 1. app.getInitialProps
  // 2. page.getInitialProps
  // 3. app.render
  // 4. page.render

  // Render app and page and get the context of the page with collected side effects.
  const sheets = new ServerStyleSheets();
  const originalRenderPage = ctx.renderPage;

  userAgent = ctx?.req?.headers["user-agent"];

  ctx.renderPage = () =>
    originalRenderPage({
      enhanceApp: (App) => (props) => sheets.collect(<App {...props} />),
    });

  const initialProps = await Document.getInitialProps(ctx);

  return {
    ...initialProps,
    // Styles fragment is rendered after the app and page rendering finish.
    styles: [
      ...React.Children.toArray(initialProps.styles),
      sheets.getStyleElement(),
    ],
  };
};

export default MyDocument;
